@@ -76,9 +76,11 @@ import Ganeti.Errors
 import Ganeti.JSON
 import Ganeti.OpParams (pTagsObject)
 import Ganeti.OpCodes
+import Ganeti.Runtime
 import qualified Ganeti.Query.Language as Qlang
 import Ganeti.THH
 import Ganeti.Types
+import Ganeti.Utils
 
 -- * Utility functions
 
@@ -119,6 +121,11 @@ $(genLuxiOp "LuxiOp"
      , simpleField "fields" [t| [String] |]
      , simpleField "lock"   [t| Bool     |]
      ])
+  , (luxiReqQueryNetworks,
+     [ simpleField "names"  [t| [String] |]
+     , simpleField "fields" [t| [String] |]
+     , simpleField "lock"   [t| Bool     |]
+     ])
   , (luxiReqQueryInstances,
      [ simpleField "names"  [t| [String] |]
      , simpleField "fields" [t| [String] |]
@@ -217,10 +224,12 @@ getClient path = do
   return Client { socket=h, rbuf=rf }
 
 -- | Creates and returns a server endpoint.
-getServer :: FilePath -> IO S.Socket
-getServer path = do
+getServer :: Bool -> FilePath -> IO S.Socket
+getServer setOwner path = do
   s <- S.socket S.AF_UNIX S.Stream S.defaultProtocol
   S.bindSocket s (S.SockAddrUnix path)
+  when setOwner . setOwnerAndGroupFromNames path GanetiLuxid $
+    ExtraGroup DaemonsGroup
   S.listen s 5 -- 5 is the max backlog
   return s
 
@@ -341,6 +350,9 @@ decodeCall (LuxiCall call args) =
               return $ QueryGroups names fields locking
     ReqQueryClusterInfo ->
               return QueryClusterInfo
+    ReqQueryNetworks -> do
+              (names, fields, locking) <- fromJVal args
+              return $ QueryNetworks names fields locking
     ReqQuery -> do
               (what, fields, qfilter) <- fromJVal args
               return $ Query what fields qfilter
