@@ -1,7 +1,7 @@
 #!/usr/bin/python
 #
 
-# Copyright (C) 2010, 2012 Google Inc.
+# Copyright (C) 2010, 2012, 2013 Google Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -37,7 +37,8 @@ from ganeti import netutils
 import testutils
 
 
-def _RunUpgrade(path, dry_run, no_verify, ignore_hostname=True):
+def _RunUpgrade(path, dry_run, no_verify, ignore_hostname=True,
+                downgrade=False):
   cmd = [sys.executable, "%s/tools/cfgupgrade" % testutils.GetSourceDir(),
          "--debug", "--force", "--path=%s" % path, "--confdir=%s" % path]
 
@@ -47,6 +48,8 @@ def _RunUpgrade(path, dry_run, no_verify, ignore_hostname=True):
     cmd.append("--dry-run")
   if no_verify:
     cmd.append("--no-verify")
+  if downgrade:
+    cmd.append("--downgrade")
 
   result = utils.RunCmd(cmd, cwd=os.getcwd())
   if result.failed:
@@ -75,6 +78,9 @@ class TestCfgupgrade(unittest.TestCase):
   def _LoadConfig(self):
     return serializer.LoadJson(utils.ReadFile(self.config_path))
 
+  def _LoadTestDataConfig(self, filename):
+    return serializer.LoadJson(testutils.ReadTestData(filename))
+
   def _CreateValidConfigDir(self):
     utils.WriteFile(self.noded_cert_path, data="")
     utils.WriteFile(self.known_hosts_path, data="")
@@ -137,6 +143,10 @@ class TestCfgupgrade(unittest.TestCase):
     utils.WriteFile(self.config_path, data=serializer.DumpJson({}))
     self.assertRaises(Exception, _RunUpgrade, self.tmpdir, False, True)
 
+  def _TestUpgradeFromFile(self, filename, dry_run):
+    cfg = self._LoadTestDataConfig(filename)
+    self._TestUpgradeFromData(cfg, dry_run)
+
   def _TestSimpleUpgrade(self, from_version, dry_run,
                          file_storage_dir=None,
                          shared_file_storage_dir=None):
@@ -153,6 +163,11 @@ class TestCfgupgrade(unittest.TestCase):
       "instances": {},
       "nodegroups": {},
       }
+    self._TestUpgradeFromData(cfg, dry_run)
+
+  def _TestUpgradeFromData(self, cfg, dry_run):
+    assert "version" in cfg
+    from_version = cfg["version"]
     self._CreateValidConfigDir()
     utils.WriteFile(self.config_path, data=serializer.DumpJson(cfg))
 
@@ -344,9 +359,59 @@ class TestCfgupgrade(unittest.TestCase):
   def testUpgradeFrom_2_6(self):
     self._TestSimpleUpgrade(constants.BuildVersion(2, 6, 0), False)
 
+  def testUpgradeFrom_2_7(self):
+    self._TestSimpleUpgrade(constants.BuildVersion(2, 7, 0), False)
+
+  def testUpgradeFullConfigFrom_2_7(self):
+    self._TestUpgradeFromFile("cluster_config_2.7.json", False)
+
   def testUpgradeCurrent(self):
     self._TestSimpleUpgrade(constants.CONFIG_VERSION, False)
 
+  def _RunDowngradeUpgrade(self):
+    oldconf = self._LoadConfig()
+    _RunUpgrade(self.tmpdir, False, True, downgrade=True)
+    _RunUpgrade(self.tmpdir, False, True)
+    newconf = self._LoadConfig()
+    self.assertEqual(oldconf, newconf)
+
+  def testDowngrade(self):
+    self._TestSimpleUpgrade(constants.CONFIG_VERSION, False)
+    self._RunDowngradeUpgrade()
+
+  def testDowngradeFullConfig(self):
+    """Test for upgrade + downgrade combination."""
+    # This test can work only with the previous version of a configuration!
+    # For 2.7, downgrading returns the original file only if group policies
+    # don't override instance specs, so we need to use an ad-hoc configuration.
+    oldconfname = "cluster_config_downgraded_2.7.json"
+    self._TestUpgradeFromFile(oldconfname, False)
+    _RunUpgrade(self.tmpdir, False, True, downgrade=True)
+    oldconf = self._LoadTestDataConfig(oldconfname)
+    newconf = self._LoadConfig()
+    self.assertEqual(oldconf, newconf)
+
+  def testDowngradeFullConfigBackwardFrom_2_7(self):
+    """Test for upgrade + downgrade + upgrade combination."""
+    self._TestUpgradeFromFile("cluster_config_2.7.json", False)
+    self._RunDowngradeUpgrade()
+
+  def _RunDowngradeTwice(self):
+    """Make sure that downgrade is idempotent."""
+    _RunUpgrade(self.tmpdir, False, True, downgrade=True)
+    oldconf = self._LoadConfig()
+    _RunUpgrade(self.tmpdir, False, True, downgrade=True)
+    newconf = self._LoadConfig()
+    self.assertEqual(oldconf, newconf)
+
+  def testDowngradeTwice(self):
+    self._TestSimpleUpgrade(constants.CONFIG_VERSION, False)
+    self._RunDowngradeTwice()
+
+  def testDowngradeTwiceFullConfigFrom_2_7(self):
+    self._TestUpgradeFromFile("cluster_config_2.7.json", False)
+    self._RunDowngradeTwice()
+
   def testUpgradeDryRunFrom_2_0(self):
     self._TestSimpleUpgrade(constants.BuildVersion(2, 0, 0), True)
 
@@ -371,6 +436,12 @@ class TestCfgupgrade(unittest.TestCase):
   def testUpgradeCurrentDryRun(self):
     self._TestSimpleUpgrade(constants.CONFIG_VERSION, True)
 
+  def testDowngradeDryRun(self):
+    self._TestSimpleUpgrade(constants.CONFIG_VERSION, False)
+    oldconf = self._LoadConfig()
+    _RunUpgrade(self.tmpdir, True, True, downgrade=True)
+    newconf = self._LoadConfig()
+    self.assertEqual(oldconf["version"], newconf["version"])
 
 if __name__ == "__main__":
   testutils.GanetiTestProgram()
